/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMXULLabeledControlEl.idl"

[scriptable, uuid(d56f1f8f-fc4e-4650-9a85-25108bbd1980)]
interface nsIDOMXULButtonElement : nsIDOMXULLabeledControlElement {
  const short CHECKSTATE_UNCHECKED = 0;
  const short CHECKSTATE_CHECKED = 1;
  const short CHECKSTATE_MIXED = 2;

  attribute DOMString type;
  attribute DOMString dlgType;

  // For buttons of type="menu" only.
  attribute boolean open;
  
  // For buttons of type="checkbox" only.
  attribute boolean checked;
  attribute long checkState;
  attribute boolean autoCheck;

  // For buttons of type="radio" only.
  attribute DOMString group;
};

